#include <NDS/NDS.h>
#include <NDS/ARM9/rand.h>
#include <NDS/ARM9/console.h> //basic print funcionality

#define PT1_CR1    (*(vuint16*)0x040001A0)
#define PT1_DATA   (*(vuint16*)0x040001A2)
#define PT1_WaitBusy()   while (PT1_CR1 & /*BUSY*/0x80); 

u8 image[ 96*72*2 +100 ];


u8 SPI(u8 d)
{
	u8 res;
	PT1_DATA = d;
	PT1_WaitBusy();
	res=PT1_DATA;
	return res;
}

u8 SPI_bit()
{
	static u8 cnt=0;
	static u8 d, result;
	if (cnt==0) {
		PT1_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | 1;  // 1 ... SPI-clock-speed==2MHz?
		d=SPI(0x00); cnt=8;
	}
	if (d & 0x80) { result=0x01; } else { result=0x00; }
	d<<=1;
	cnt--;
	
	return result;
}

void Scan()
{
	u32 bts;
	u8 cnt,d;
	cnt=0;
	bts=0;
//	while (1) {
//		d=SPI_bit();
//		if (d==0x01) {
//			cnt++; if (cnt>=100) { break; }
//		} else {
//			cnt=0;
//		}
//	}
	while (1) {
		d=SPI_bit();
		bts<<=1;
		bts |= d;
		if (bts==0xAA55FFD8) { break; }
	}
	for (cnt=0;cnt<26*8;cnt++) {
		d=SPI_bit();
	}
}


u8 Read_byte()
{
	u8 i,d;
	d=0;
	for (i=0;i<8;i++) {
		d<<=1;
		d |= SPI_bit();
	}
	return d;
}

void Read_image()
{
	u16 i;
	for (i=0;i<96*72*2;i++) {
		image[i]=Read_byte();
	}
}


int main()
{	
	u16 key_st,prev_key;
	int i,cnt;
	u8 d1,d2;
	u16 x,y,pos;
	uint16 *vbuf;
	int red,green,blue;
	int Y,U,V,R,G,B;
	
	
	prev_key=0;
	powerON(POWER_LCD_TOP | POWER_LCD_BOTTOM | POWER_2D | POWER_2D_SUB |
			POWER_3D_CORE | POWER_MATRIX | POWER_SWAP_LCDS);
	videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE | ENABLE_3D);
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE); //sub bg 0 will be used to print text
	vramSetBankC(VRAM_C_SUB_BG); 
	
	videoSetMode(MODE_FB0);
	vramSetBankA(VRAM_A_LCD);

	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);//by default font will be rendered with color 255
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
	
	WAIT_CR &= ~0x0880;	// DS Card access ARM9:bit11=0   GBA Cart access ARM9:bit7=0
	
	consolePrintSet(0,1);
	consolePrintf("<<< TREVA TEST >>>\n");
	
	cnt=0;
	while (1) {
		
		Scan(); // 摜f[^̓o
		d1=Read_byte(); // 0xAAXLbv
		d2=Read_byte(); // 0x55XLbv
		//consolePrintSet(0,2); consolePrintf("%02X %02X\n",d1,d2);
		Read_image();
		
		cnt++;
		consolePrintSet(0,3); consolePrintf("(%d)\n",cnt);
		
		
		for (y=0;y<72;y++) {
			vbuf=VRAM_A + y*SCREEN_WIDTH*2;
			for (x=0;x<96;x++) {
				pos = ( (71-y)*96 + (95-x) ) *2;
				if (x%2==0) {
					Y = image[pos+1];
					U = image[pos+2];
					V = image[pos];
				} else {
					Y = image[pos+1];
					U = image[pos];
					V = image[pos-2];
				}
				
				U = U -128;
				V = V -128;
				
				R = U+Y;
				G = (98*Y - 53*U - 19*V)/100;
				B = V+Y;
				
				R = R*31/255;
				G = G*31/255;
				B = B*31/255;
				
				if (R<0) { R=0; }
				if (G<0) { G=0; }
				if (B<0) { B=0; }
				
				if (R>31) { R=31; }
				if (G>31) { G=31; }
				if (B>31) { B=31; }
				
				red = (int)R;
				green = (int)G;
				blue = (int)B;
				
				*(vbuf+x*2) = RGB15( blue,green,red );
				*(vbuf+x*2+1) = RGB15( blue,green,red );
				*(vbuf+x*2+SCREEN_WIDTH) = RGB15( blue,green,red );
				*(vbuf+x*2+SCREEN_WIDTH+1) = RGB15( blue,green,red );
			}
		}
		
		
		key_st = KEYS;
		if (key_st != prev_key) {
			//if ( ( !(key_st & KEY_LEFT) )&&( !(key_st & KEY_RIGHT)) &&( !(key_st & KEY_START) ) ) { break; }
		}
		prev_key = key_st;
		for (i=0;i<100;i++) { ; } //wait
	}
	
	
	return 0;
}//end main 

